# Formable

A form builder module for [Drupal][1], fork of [core contact module][3].

## Why?

In short: I don't like the [core contact][3] or [webform][9] modules. Improving the contact module was not possible because of heavy, breaking changes. Improving the webform module was not possible because of privacy issues and therefore trust issues. So I decided to go the hard route.

Related, long forum post: [State of form builder modules - or: How to build (contact) forms without webform?][8]

See also docs/decisions folder in the [formable source repository][12].

## Features/TODO

* removed from core [contact][3]
  * [x] disabled hard coded `/contact`, `/contact/{form}` and `/{lang}/contact/{form}` routes
  * [x] disabled hard coded footer menu item
  * [x] disabled personal user forms
  * [x] disabled auto-reply (to prevent spam abuse)
  * [x] disabled "Send yourself a copy"
  * [ ] disabled redirect to front page
  * [x] dropped base field definitions (name, mail, subject, message)
* [x] added option to expose forms under custom route
* implemented from [contact_block][5]
  * [x] added block usage
* implemented from [contact_storage][11] module
  * [x] store messages by default
  * user interface to list/view/delete stored messages
    * [x] list
    * [ ] view single message (with more details, eventually with field formatters)
    * [x] delete
    * [ ] multiple delete
  * [ ] custom text for submit button
  * [ ] clone form
  * [ ] disable form (currently disabled if invalid)
* [x] option to disable storing messages
* implemented from [contact_storage_dm][6]
  * [x] disable sending mails (by leaving recipients field empty)
* implement [a11y_form_helpers][7]
  * [ ] aria-describedby for error messages
  * [ ] ...
* [ ] implement validation
* [x] implement honeypot
* [x] implement simple captcha
* missing changes from core after hard fork
  * [ ] [Determine which core config entity methods should be config actions](https://git.drupalcode.org/project/drupal/-/commit/d9a1ec2d4e0356d20825f4962e8d71dea02ccb02#f764c91d03b672b82b68fc01930d56212c33fe3f)
* [ ] implement REST for messages

TODO: collect more features from existing modules

## Installation

install drupal and drush:

```bash
mkdir -p my-project && cd my-project
composer create-project drupal/recommended-project .
composer require drush/drush
# call `./vendor/bin/drush` instead of `drush` or
# add drush to $PATH - see https://www.drush.org/12.x/install/
drush site:install
```

install and enable module:

```bash
composer require drupal/formable
drush pm:install formable
```

grant permissions to view forms as anonymous user:

```bash
drush role:perm:add anonymous 'access site-wide formable form,access block-view formable form'
```

## Uninstallation

```bash
# The module can't be uninstalled without removing associated entities from
# the database beforehand
drush entity:delete formable_message
# uninstall module
drush pm:uninstall formable
# delete module
composer remove drupal/formable
```

## Development

see [docs/development.md][13]

## Copyright and License

TODO: Should I omit my name from the copyright notice (hard fork, many contributors...)?

Copyright 2024 Raffael Jesche under the [GPL-2.0-or-later][2] license.

See `LICENSE.txt` for more information.

[1]: https://www.drupal.org/
[2]: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
[3]: https://www.drupal.org/docs/8/core/modules/contact/overview
[5]: https://www.drupal.org/project/contact_block
[6]: https://www.drupal.org/project/contact_storage_dm
[7]: https://www.drupal.org/project/a11y_form_helpers
[8]: https://www.drupal.org/forum/support/post-installation/2024-02-20/state-of-form-builder-modules-or-how-to-build-contact-forms-without-webform
[9]: https://www.drupal.org/project/webform
[11]: https://www.drupal.org/project/contact_storage
[12]: https://git.drupalcode.org/project/formable
[13]: https://git.drupalcode.org/project/formable/-/tree/0.x/docs/development.md
