<?php

namespace Drupal\formable;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Defines the access control handler for the form entity type.
 *
 * @see \Drupal\formable\Entity\FormableForm.
 */
class FormableFormAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    if ($operation == 'view') {
      if (!$entity->isValid()) {
        // allow admins previewing the form even when it is invalid
        return AccessResult::allowedIfHasPermission($account, 'administer formable forms')
          ->andIf(AccessResult::allowedIfHasPermission($account, 'access site-wide formable form'));
      }
      return AccessResult::allowedIfHasPermission($account, 'access site-wide formable form');
    }
    if ($operation == 'block-view') {
      if (!$entity->isValid()) {
        // allow admins previewing the block even when it is invalid
        return AccessResult::allowedIfHasPermission($account, 'administer formable forms')
          ->andIf(AccessResult::allowedIfHasPermission($account, 'access block-view formable form'));
      }
      return AccessResult::allowedIfHasPermission($account, 'access block-view formable form');
    }
    if ($operation == 'delete' || $operation == 'update') {
      return AccessResult::allowedIfHasPermission($account, 'administer formable forms');
    }

    return parent::checkAccess($entity, $operation, $account);
  }

}
