<?php

namespace Drupal\formable;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Defines a class to build a listing of form entities.
 *
 * TODO: list more useful data, e. g.:
 * - is used in a block
 * - number of fields
 *
 * @see \Drupal\formable\Entity\FormableForm
 */
class FormableFormListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['form'] = t('Form');
    $header['is_valid'] = t('Valid');
    $header['recipients'] = t('Recipients');
    $header['store_messages'] = t('Store messages');
    $header['route'] = t('Route');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {

    $linkToMessages = $this->getLinkToMessages($entity, $entity->label());
    $row['form'] = $linkToMessages->toString();

    $row['is_valid'] = $entity->isValid() ? t('Yes') : t('No');
    $row['recipients']['data'] = [
      '#theme' => 'item_list',
      '#items' => $entity->getRecipients(),
      '#context' => ['list_style' => 'comma-list'],
    ];

    $row['store_messages'] = $entity->getStoreMessages() ? t('Yes') : t('No');

    $routeLink = $entity->getRouteLink();
    $row['route'] = $routeLink ? $routeLink->toString() : $entity->getRoute();
    return $row + parent::buildRow($entity);
  }

  /**
   * Add 'list_messages' operation to drop-down menu
   *
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {

    $operations = parent::getDefaultOperations($entity);

    $operations['list_messages'] = [
      'title' => t('Messages'),
      'weight' => 1,
      'url' => $this->getUrlToMessages($entity),
    ];

    return $operations;
  }

  /**
   * Returns url object to messages
   */
  public function getUrlToMessages(EntityInterface $entity) {
    return new Url(
      'entity.formable_message.collection',
      ['formable_form' => $entity->id()]
    );
  }

  /**
   * Returns link object to messages
   */
  public function getLinkToMessages(EntityInterface $entity, string $label = null) {
    $urlToMessages = $this->getUrlToMessages($entity);
    if (!$label) $label = $this->t('Messages');
    return new Link($label, $urlToMessages);
  }

}
