<?php

namespace Drupal\formable\Routing;

use Symfony\Component\Routing\Route;

/**
 * Defines dynamic routes.
 */
class RouteSubscriber {

  /**
   * Provides dynamic routes.
   *
   * Routes are language specific,
   * e. g.: `/en/contact-form` and `/de/kontaktformular`
   *
   * Untranslated routes fall back to the default route
   */
  public function routes() {

    $routes = [];

    // TODO: use dependency injection
    $entities = \Drupal::entityTypeManager()->getStorage('formable_form')->loadMultiple();

    $languageManager = \Drupal::languageManager();
    $languageList    = $languageManager->getLanguages();
    $defaultLanguage = $languageManager->getDefaultLanguage()->getId();

    foreach ($entities as $entity) {

      // don't create route if form can't be submitted correctly
      if (!$entity->isValid()) continue;

      $name = $entity->id();
      $configName = 'formable.form.' . $entity->id();

      foreach ($languageList as $code => $language) {

        if ($code !== $defaultLanguage) {
          $translatedConfig = $languageManager->getLanguageConfigOverride($code, $configName);
          $route = $translatedConfig->get('route');
        } else {
          $route = $entity->getRoute();
        }

        // prevent invalid config like whitespace
        if (!is_string($route)) continue;
        $route = trim($route);

        if (empty($route)) continue;

        // TODO: more checks, if route is valid

        $routes["formable.form.{$name}.{$code}"] = new Route(
          $route,
          // defaults
          [
            '_controller' => '\Drupal\formable\Controller\FormableController::formableSitePage',
            // some magic changes that name argument to the entity
            // TODO: How does this magic work?
            'formable_form' => $name,
            'language' => $code,
          ],
          // route requirements
          [
            '_permission'  => 'access site-wide formable form',
            '_entity_access' => 'formable_form.view',
          ]
        );
      }
    }

    return $routes;
  }

}
